// данный скетч полностью настроенный и проверенный! В стоке ничего менять не надо
// все важные (и не очень) строки закемментированы для вас
// убедитесь, установлены ли библиотеки ниже, если нет - установите их
#include <PID_v1.h>
#include <LMotorController.h>
#include "I2Cdev.h"
#include "MPU6050_6Axis_MotionApps20.h"
 #if I2CDEV_IMPLEMENTATION == I2CDEV_ARDUINO_WIRE
#include "Wire.h"
#endif
#define MIN_ABS_SPEED 20
 
MPU6050 mpu;
 
// установки гироскопа (лучше в этом блоке ничего не трогать)
bool dmpReady = false; // установите true, если DMP init правильный
uint8_t mpuIntStatus; // содержит фактические данные статуса прерывания от MPU
uint8_t devStatus; // статус возврата после каждой операции устройства (0 = успех, !0 = ошибка)
uint16_t packetSize; // ожидаемый размер пакета DMP (по умолчанию - 42 bytes)
uint16_t fifoCount; // количество всех байтов, находящихся в настоящее время в FIFO (память)
uint8_t fifoBuffer[64]; // буфер хранения FIFO
 
Quaternion q;
VectorFloat gravity;
float ypr[3];
 
//PID настройки
double originalSetpoint = 178; // максимальный угол наклона, который придерживается робот (от180)
double setpoint = originalSetpoint;
double movingAngleOffset = 0.1; // угловое ускорение (чем меньше, чем выше робот)
double input, output;
 
//отрегулируйте эти коэффициенты в соответствии с вашим роботом
double Kp = 50; //настройка "резкости" движений
double Kd = 1.4; //коэффициент "колебаний" робота
double Ki = 60; //при правильной настройке сократит время стабилизации робота
PID pid(&input, &output, &setpoint, Kp, Ki, Kd, DIRECT);
// ниже настройте частоты вращения правого и левого двигателей (кстановки под меня)
double motorSpeedFactorLeft = 0.7;
double motorSpeedFactorRight = 0.5;

int ENA = 5;
int IN1 = 6;
int IN2 = 7;
int IN3 = 8;
int IN4 = 9;
int ENB = 10;
LMotorController motorController(ENA, IN1, IN2, ENB, IN3, IN4, motorSpeedFactorLeft, motorSpeedFactorRight);
 
volatile bool mpuInterrupt = false; // указывает, прошел ли вывод прерывания MPU
void dmpDataReady()
{
mpuInterrupt = true;
}
 
 
void setup()
{
#if I2CDEV_IMPLEMENTATION == I2CDEV_ARDUINO_WIRE
Wire.begin();
TWBR = 24; 
#elif I2CDEV_IMPLEMENTATION == I2CDEV_BUILTIN_FASTWIRE
Fastwire::setup(400, true);
#endif
 
mpu.initialize();
 
devStatus = mpu.dmpInitialize();
 
// при необходимости поставьте здесь свои значения для гироскопа (регулировка чувствительноси)
mpu.setXGyroOffset(220);
mpu.setYGyroOffset(76);
mpu.setZGyroOffset(-85);
mpu.setZAccelOffset(1788);
// ниже пояснения, оставляем все как есть
if (devStatus == 0)
{
// включите DMP, теперь, когда он готов
mpu.setDMPEnabled(true);
 
// включить обнаружение прерываний Arduino
attachInterrupt(0, dmpDataReady, RISING);
mpuIntStatus = mpu.getIntStatus();
 
// установите DMP Ready true, чтобы функция main loop() знала, что ее можно использовать
dmpReady = true;
 
// получить ожидаемый размер пакета DMP для последующего сравнения
packetSize = mpu.dmpGetFIFOPacketSize();
 
pid.SetMode(AUTOMATIC);
pid.SetSampleTime(10);
pid.SetOutputLimits(-255, 255); 
}
else
{
// Ошибка!
// 1 = ошибка загрузки исходной памяти
// 2 = не удалось обновить конфигурацию DMP
Serial.print(F("DMP Initialization failed (code "));
Serial.print(devStatus);
Serial.println(F(")"));
}
}
 
 void loop()
{
// если программирование завершилось неудачно, не пытайтесь ничего делать
if (!dmpReady) return;
 
// дождитесь прерывания MPU
while (!mpuInterrupt && fifoCount < packetSize)
{
pid.Compute();
motorController.move(output, MIN_ABS_SPEED);
 
}
 
mpuInterrupt = false;
mpuIntStatus = mpu.getIntStatus();
 
// получать текущее количество FIFO
fifoCount = mpu.getFIFOCount();
 
// проверьте переполнение
if ((mpuIntStatus & 0x10) || fifoCount == 1024)
{
// сброс, чтобы мы могли продолжить
mpu.resetFIFO();
Serial.println(F("FIFO overflow!"));
 
}
else if (mpuIntStatus & 0x02)
{
while (fifoCount < packetSize) fifoCount = mpu.getFIFOCount();
 
// читать пакет из FIFO
mpu.getFIFOBytes(fifoBuffer, packetSize);
 
fifoCount -= packetSize;
 
mpu.dmpGetQuaternion(&q, fifoBuffer);
mpu.dmpGetGravity(&gravity, &q);
mpu.dmpGetYawPitchRoll(ypr, &q, &gravity);
input = ypr[1] * 180/M_PI + 180;
}
}
